//+------------------------------------------------------------------+
//|                                                  TestGlobals.mq5 |
//|                                  Copyright 2024, MetaQuotes Ltd. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2024, MetaQuotes Ltd."
#property link      "https://www.mql5.com"
#property version   "1.00"


class CEATestModule{
public:
   const string m_strSymbol;
   string m_strTestModuleID;
   
   CEATestModule(string strSymbol): m_strSymbol(strSymbol){
      m_strTestModuleID = StringFormat("%s_%I64u", m_strSymbol, &this);
   };
   
   void AssertConsistency(){
      int iPos = StringFind(m_strTestModuleID, "_");
      string strCorrectSymbol = StringSubstr(m_strTestModuleID, 0, iPos); 
      if(m_strSymbol != strCorrectSymbol){
         DebugBreak();
         PrintFormat("Inconsistency detected: CorrectSymbol = %s; CurrentSymbol = %s", strCorrectSymbol, m_strSymbol);
      }
   }
};

CEATestModule* g_objTestModule;
string g_strSymbol;

//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit(){
   if(CheckPointer(g_objTestModule) == POINTER_DYNAMIC)
      g_objTestModule.AssertConsistency();
   else   
      g_objTestModule = new CEATestModule(_Symbol);
   
   g_strSymbol = _Symbol;
   
   EventSetTimer(1);
   OnTimer();
   
   return(INIT_SUCCEEDED);
}


void OnTimer(){
   datetime dtCurrentTime = TimeLocal();
   
   static datetime s_dtStartTime = TimeLocal();
   static bool s_bSymbolChanged = false;
   if(!s_bSymbolChanged && dtCurrentTime - s_dtStartTime >= 3){
      ChartSetSymbolPeriod(ChartID(), "AUDCAD", _Period); //Replace "AUDCAD" with any symbol in your MarketWatch
      s_bSymbolChanged = true;
   }
}


//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick(){  
}


//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason){
   EventKillTimer();
   //delete g_objTestModule; - Make sure you do not delete g_objTestModule here. It must pass through to the next _Symbol
   //In real life, the pointer should be destroyed in some global object destructor to avoid memory leaks as shown below.
}

class CGlobal{
public:
   ~CGlobal(){
      if(CheckPointer(g_objTestModule) == POINTER_DYNAMIC)
         delete g_objTestModule;
   }
};
CGlobal g_objGlobal;